@@ -3,27 +3,53 @@
 
 """HTML reporting for coverage.py."""
 
+from __future__ import annotations
+
 import datetime
 import json
 import os
 import re
 import shutil
-import types
+
+from dataclasses import dataclass
+from typing import Any, Dict, Iterable, List, Optional, Tuple, TYPE_CHECKING, cast
 
 import coverage
-from coverage.data import add_data_to_hash
+from coverage.data import CoverageData, add_data_to_hash
 from coverage.exceptions import NoDataError
 from coverage.files import flat_rootname
 from coverage.misc import ensure_dir, file_be_gone, Hasher, isolate_module, format_local_datetime
 from coverage.misc import human_sorted, plural
 from coverage.report import get_analysis_to_report
-from coverage.results import Numbers
+from coverage.results import Analysis, Numbers
 from coverage.templite import Templite
+from coverage.types import TLineNo, TMorf
+
+
+if TYPE_CHECKING:
+    # To avoid circular imports:
+    from coverage import Coverage
+    from coverage.plugins import FileReporter
+
+    # To be able to use 3.8 typing features, and still run on 3.7:
+    from typing import TypedDict
+
+    class IndexInfoDict(TypedDict):
+        """Information for each file, to render the index file."""
+        nums: Numbers
+        html_filename: str
+        relative_filename: str
+
+    class FileInfoDict(TypedDict):
+        """Summary of the information from last rendering, to avoid duplicate work."""
+        hash: str
+        index: IndexInfoDict
+
 
 os = isolate_module(os)
 
 
-def data_filename(fname):
+def data_filename(fname: str) -> str:
     """Return the path to an "htmlfiles" data file of ours.
     """
     static_dir = os.path.join(os.path.dirname(__file__), "htmlfiles")
@@ -31,25 +57,51 @@ def data_filename(fname):
     return static_filename
 
 
-def read_data(fname):
+def read_data(fname: str) -> str:
     """Return the contents of a data file of ours."""
     with open(data_filename(fname)) as data_file:
         return data_file.read()
 
 
-def write_html(fname, html):
+def write_html(fname: str, html: str) -> None:
     """Write `html` to `fname`, properly encoded."""
     html = re.sub(r"(\A\s+)|(\s+$)", "", html, flags=re.MULTILINE) + "\n"
     with open(fname, "wb") as fout:
         fout.write(html.encode('ascii', 'xmlcharrefreplace'))
 
 
+@dataclass
+class LineData:
+    """The data for each source line of HTML output."""
+    tokens: List[Tuple[str, str]]
+    number: TLineNo
+    category: str
+    statement: bool
+    contexts: List[str]
+    contexts_label: str
+    context_list: List[str]
+    short_annotations: List[str]
+    long_annotations: List[str]
+    html: str = ""
+    annotate: Optional[str] = None
+    annotate_long: Optional[str] = None
+    css_class: str = ""
+
+
+@dataclass
+class FileData:
+    """The data for each source file of HTML output."""
+    relative_filename: str
+    nums: Numbers
+    lines: List[LineData]
+
+
 class HtmlDataGeneration:
     """Generate structured data to be turned into HTML reports."""
 
     EMPTY = "(empty)"
 
-    def __init__(self, cov):
+    def __init__(self, cov: Coverage) -> None:
         self.coverage = cov
         self.config = self.coverage.config
         data = self.coverage.get_data()
@@ -59,7 +111,7 @@ class HtmlDataGeneration:
                 self.coverage._warn("No contexts were measured")
         data.set_query_contexts(self.config.report_contexts)
 
-    def data_for_file(self, fr, analysis):
+    def data_for_file(self, fr: FileReporter, analysis: Analysis) -> FileData:
         """Produce the data needed for one file's report."""
         if self.has_arcs:
             missing_branch_arcs = analysis.missing_branch_arcs()
@@ -72,7 +124,7 @@ class HtmlDataGeneration:
 
         for lineno, tokens in enumerate(fr.source_token_lines(), start=1):
             # Figure out how to mark this line.
-            category = None
+            category = ""
             short_annotations = []
             long_annotations = []
 
@@ -86,13 +138,14 @@ class HtmlDataGeneration:
                     if b < 0:
                         short_annotations.append("exit")
                     else:
-                        short_annotations.append(b)
+                        short_annotations.append(str(b))
                     long_annotations.append(fr.missing_arc_description(lineno, b, arcs_executed))
             elif lineno in analysis.statements:
                 category = 'run'
 
-            contexts = contexts_label = None
-            context_list = None
+            contexts = []
+            contexts_label = ""
+            context_list = []
             if category and self.config.show_contexts:
                 contexts = human_sorted(c or self.EMPTY for c in contexts_by_lineno.get(lineno, ()))
                 if contexts == [self.EMPTY]:
@@ -101,7 +154,7 @@ class HtmlDataGeneration:
                     contexts_label = f"{len(contexts)} ctx"
                     context_list = contexts
 
-            lines.append(types.SimpleNamespace(
+            lines.append(LineData(
                 tokens=tokens,
                 number=lineno,
                 category=category,
@@ -113,7 +166,7 @@ class HtmlDataGeneration:
                 long_annotations=long_annotations,
             ))
 
-        file_data = types.SimpleNamespace(
+        file_data = FileData(
             relative_filename=fr.relative_filename(),
             nums=analysis.numbers,
             lines=lines,
@@ -122,6 +175,15 @@ class HtmlDataGeneration:
         return file_data
 
 
+class FileToReport:
+    """A file we're considering reporting."""
+    def __init__(self, fr: FileReporter, analysis: Analysis) -> None:
+        self.fr = fr
+        self.analysis = analysis
+        self.rootname = flat_rootname(fr.relative_filename())
+        self.html_filename = self.rootname + ".html"
+
+
 class HtmlReporter:
     """HTML reporting."""
 
@@ -135,7 +197,7 @@ class HtmlReporter:
         "favicon_32.png",
     ]
 
-    def __init__(self, cov):
+    def __init__(self, cov: Coverage) -> None:
         self.coverage = cov
         self.config = self.coverage.config
         self.directory = self.config.html_dir
@@ -151,6 +213,7 @@ class HtmlReporter:
 
         title = self.config.html_title
 
+        self.extra_css: Optional[str]
         if self.config.extra_css:
             self.extra_css = os.path.basename(self.config.extra_css)
         else:
@@ -159,12 +222,14 @@ class HtmlReporter:
         self.data = self.coverage.get_data()
         self.has_arcs = self.data.has_arcs()
 
-        self.file_summaries = []
-        self.all_files_nums = []
+        self.file_summaries: List[IndexInfoDict] = []
+        self.all_files_nums: List[Numbers] = []
         self.incr = IncrementalChecker(self.directory)
         self.datagen = HtmlDataGeneration(self.coverage)
         self.totals = Numbers(precision=self.config.precision)
         self.directory_was_empty = False
+        self.first_fr = None
+        self.final_fr = None
 
         self.template_globals = {
             # Functions available in the templates.
@@ -193,7 +258,7 @@ class HtmlReporter:
         self.pyfile_html_source = read_data("pyfile.html")
         self.source_tmpl = Templite(self.pyfile_html_source, self.template_globals)
 
-    def report(self, morfs):
+    def report(self, morfs: Optional[Iterable[TMorf]]) -> float:
         """Generate an HTML report for `morfs`.
 
         `morfs` is a list of modules or file names.
@@ -204,22 +269,52 @@ class HtmlReporter:
         self.incr.read()
         self.incr.check_global_data(self.config, self.pyfile_html_source)
 
-        # Process all the files.
+        # Process all the files. For each page we need to supply a link
+        # to the next and previous page.
+        files_to_report = []
+
         for fr, analysis in get_analysis_to_report(self.coverage, morfs):
-            self.html_file(fr, analysis)
+            ftr = FileToReport(fr, analysis)
+            should = self.should_report_file(ftr)
+            if should:
+                files_to_report.append(ftr)
+            else:
+                file_be_gone(os.path.join(self.directory, ftr.html_filename))
+
+        for i, ftr in enumerate(files_to_report):
+            if i == 0:
+                prev_html = "index.html"
+            else:
+                prev_html = files_to_report[i - 1].html_filename
+            if i == len(files_to_report) - 1:
+                next_html = "index.html"
+            else:
+                next_html = files_to_report[i + 1].html_filename
+            self.write_html_file(ftr, prev_html, next_html)
 
         if not self.all_files_nums:
             raise NoDataError("No data to report.")
 
-        self.totals = sum(self.all_files_nums)
+        self.totals = cast(Numbers, sum(self.all_files_nums))
 
         # Write the index file.
-        self.index_file()
+        if files_to_report:
+            first_html = files_to_report[0].html_filename
+            final_html = files_to_report[-1].html_filename
+        else:
+            first_html = final_html = "index.html"
+        self.index_file(first_html, final_html)
 
         self.make_local_static_report_files()
         return self.totals.n_statements and self.totals.pc_covered
 
-    def make_local_static_report_files(self):
+    def make_directory(self) -> None:
+        """Make sure our htmlcov directory exists."""
+        ensure_dir(self.directory)
+        if not os.listdir(self.directory):
+            self.directory_was_empty = True
+
+    def make_local_static_report_files(self) -> None:
         """Make local instances of static files for HTML report."""
         # The files we provide must always be copied.
         for static in self.STATIC_FILES:
@@ -234,19 +329,13 @@ class HtmlReporter:
 
         # The user may have extra CSS they want copied.
         if self.extra_css:
+            assert self.config.extra_css is not None
             shutil.copyfile(self.config.extra_css, os.path.join(self.directory, self.extra_css))
 
-    def html_file(self, fr, analysis):
-        """Generate an HTML file for one source file."""
-        rootname = flat_rootname(fr.relative_filename())
-        html_filename = rootname + ".html"
-        ensure_dir(self.directory)
-        if not os.listdir(self.directory):
-            self.directory_was_empty = True
-        html_path = os.path.join(self.directory, html_filename)
-
+    def should_report_file(self, ftr: FileToReport) -> bool:
+        """Determine if we'll report this file."""
         # Get the numbers for this file.
-        nums = analysis.numbers
+        nums = ftr.analysis.numbers
         self.all_files_nums.append(nums)
 
         if self.skip_covered:
@@ -255,36 +344,40 @@ class HtmlReporter:
             no_missing_branches = (nums.n_partial_branches == 0)
             if no_missing_lines and no_missing_branches:
                 # If there's an existing file, remove it.
-                file_be_gone(html_path)
                 self.skipped_covered_count += 1
-                return
+                return False
 
         if self.skip_empty:
             # Don't report on empty files.
             if nums.n_statements == 0:
-                file_be_gone(html_path)
                 self.skipped_empty_count += 1
-                return
+                return False
+
+        return True
+
+    def write_html_file(self, ftr: FileToReport, prev_html: str, next_html: str) -> None:
+        """Generate an HTML file for one source file."""
+        self.make_directory()
 
         # Find out if the file on disk is already correct.
-        if self.incr.can_skip_file(self.data, fr, rootname):
-            self.file_summaries.append(self.incr.index_info(rootname))
+        if self.incr.can_skip_file(self.data, ftr.fr, ftr.rootname):
+            self.file_summaries.append(self.incr.index_info(ftr.rootname))
             return
 
         # Write the HTML page for this file.
-        file_data = self.datagen.data_for_file(fr, analysis)
+        file_data = self.datagen.data_for_file(ftr.fr, ftr.analysis)
         for ldata in file_data.lines:
             # Build the HTML for the line.
-            html = []
+            html_parts = []
             for tok_type, tok_text in ldata.tokens:
                 if tok_type == "ws":
-                    html.append(escape(tok_text))
+                    html_parts.append(escape(tok_text))
                 else:
                     tok_html = escape(tok_text) or '&nbsp;'
-                    html.append(
+                    html_parts.append(
                         f'<span class="{tok_type}">{tok_html}</span>'
                     )
-            ldata.html = ''.join(html)
+            ldata.html = ''.join(html_parts)
 
             if ldata.short_annotations:
                 # 202F is NARROW NO-BREAK SPACE.
@@ -313,23 +406,31 @@ class HtmlReporter:
 
             css_classes = []
             if ldata.category:
-                css_classes.append(self.template_globals['category'][ldata.category])
+                css_classes.append(
+                    self.template_globals['category'][ldata.category]   # type: ignore[index]
+                )
             ldata.css_class = ' '.join(css_classes) or "pln"
 
-        html = self.source_tmpl.render(file_data.__dict__)
+        html_path = os.path.join(self.directory, ftr.html_filename)
+        html = self.source_tmpl.render({
+            **file_data.__dict__,
+            'prev_html': prev_html,
+            'next_html': next_html,
+        })
         write_html(html_path, html)
 
         # Save this file's information for the index file.
-        index_info = {
-            'nums': nums,
-            'html_filename': html_filename,
-            'relative_filename': fr.relative_filename(),
+        index_info: IndexInfoDict = {
+            'nums': ftr.analysis.numbers,
+            'html_filename': ftr.html_filename,
+            'relative_filename': ftr.fr.relative_filename(),
         }
         self.file_summaries.append(index_info)
-        self.incr.set_index_info(rootname, index_info)
+        self.incr.set_index_info(ftr.rootname, index_info)
 
-    def index_file(self):
+    def index_file(self, first_html: str, final_html: str) -> None:
         """Write the index.html file for this report."""
+        self.make_directory()
         index_tmpl = Templite(read_data("index.html"), self.template_globals)
 
         skipped_covered_msg = skipped_empty_msg = ""
@@ -345,6 +446,8 @@ class HtmlReporter:
             'totals': self.totals,
             'skipped_covered_msg': skipped_covered_msg,
             'skipped_empty_msg': skipped_empty_msg,
+            'first_html': first_html,
+            'final_html': final_html,
         })
 
         index_file = os.path.join(self.directory, "index.html")
@@ -361,7 +464,6 @@ class IncrementalChecker:
     STATUS_FILE = "status.json"
     STATUS_FORMAT = 2
 
-    #           pylint: disable=wrong-spelling-in-comment,useless-suppression
     #  The data looks like:
     #
     #  {
@@ -389,16 +491,16 @@ class IncrementalChecker:
     #      }
     #  }
 
-    def __init__(self, directory):
+    def __init__(self, directory: str) -> None:
         self.directory = directory
         self.reset()
 
-    def reset(self):
+    def reset(self) -> None:
         """Initialize to empty. Causes all files to be reported."""
         self.globals = ''
-        self.files = {}
+        self.files: Dict[str, FileInfoDict] = {}
 
-    def read(self):
+    def read(self) -> None:
         """Read the information we stored last time."""
         usable = False
         try:
@@ -423,12 +525,13 @@ class IncrementalChecker:
         else:
             self.reset()
 
-    def write(self):
+    def write(self) -> None:
         """Write the current status."""
         status_file = os.path.join(self.directory, self.STATUS_FILE)
         files = {}
         for filename, fileinfo in self.files.items():
-            fileinfo['index']['nums'] = fileinfo['index']['nums'].init_args()
+            index = fileinfo['index']
+            index['nums'] = index['nums'].init_args()   # type: ignore[typeddict-item]
             files[filename] = fileinfo
 
         status = {
@@ -440,7 +543,7 @@ class IncrementalChecker:
         with open(status_file, "w") as fout:
             json.dump(status, fout, separators=(',', ':'))
 
-    def check_global_data(self, *data):
+    def check_global_data(self, *data: Any) -> None:
         """Check the global data that can affect incremental reporting."""
         m = Hasher()
         for d in data:
@@ -450,7 +553,7 @@ class IncrementalChecker:
             self.reset()
             self.globals = these_globals
 
-    def can_skip_file(self, data, fr, rootname):
+    def can_skip_file(self, data: CoverageData, fr: FileReporter, rootname: str) -> bool:
         """Can we skip reporting this file?
 
         `data` is a CoverageData object, `fr` is a `FileReporter`, and
@@ -470,26 +573,26 @@ class IncrementalChecker:
             self.set_file_hash(rootname, this_hash)
             return False
 
-    def file_hash(self, fname):
+    def file_hash(self, fname: str) -> str:
         """Get the hash of `fname`'s contents."""
-        return self.files.get(fname, {}).get('hash', '')
+        return self.files.get(fname, {}).get('hash', '')    # type: ignore[call-overload]
 
-    def set_file_hash(self, fname, val):
+    def set_file_hash(self, fname: str, val: str) -> None:
         """Set the hash of `fname`'s contents."""
-        self.files.setdefault(fname, {})['hash'] = val
+        self.files.setdefault(fname, {})['hash'] = val      # type: ignore[typeddict-item]
 
-    def index_info(self, fname):
+    def index_info(self, fname: str) -> IndexInfoDict:
         """Get the information for index.html for `fname`."""
-        return self.files.get(fname, {}).get('index', {})
+        return self.files.get(fname, {}).get('index', {})   # type: ignore
 
-    def set_index_info(self, fname, info):
+    def set_index_info(self, fname: str, info: IndexInfoDict) -> None:
         """Set the information for index.html for `fname`."""
-        self.files.setdefault(fname, {})['index'] = info
+        self.files.setdefault(fname, {})['index'] = info    # type: ignore[typeddict-item]
 
 
 # Helpers for templates and generating HTML
 
-def escape(t):
+def escape(t: str) -> str:
     """HTML-escape the text in `t`.
 
     This is only suitable for HTML text, not attributes.
@@ -499,6 +602,6 @@ def escape(t):
     return t.replace("&", "&amp;").replace("<", "&lt;")
 
 
-def pair(ratio):
+def pair(ratio: Tuple[int, int]) -> str:
     """Format a pair of numbers so JavaScript can read them in an attribute."""
     return "%s %s" % ratio
