@@ -3,21 +3,25 @@
 
 """Tests about understanding how third-party code is installed."""
 
+from __future__ import annotations
+
 import os
 import os.path
 import shutil
-import sys
+
+from pathlib import Path
+from typing import Iterator, cast
 
 import pytest
 
 from coverage import env
 
-from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import change_dir, make_file, nice_file
+from tests.coveragetest import CoverageTest, COVERAGE_INSTALL_ARGS
+from tests.helpers import change_dir, make_file
 from tests.helpers import re_lines, run_command
 
 
-def run_in_venv(cmd):
+def run_in_venv(cmd: str) -> str:
     r"""Run `cmd` in the virtualenv at `venv`.
 
     The first word of the command will be adjusted to run it from the
@@ -38,7 +42,7 @@ def run_in_venv(cmd):
 
 
 @pytest.fixture(scope="session", name="venv_world")
-def venv_world_fixture(tmp_path_factory):
+def venv_world_fixture(tmp_path_factory: pytest.TempPathFactory) -> Path:
     """Create a virtualenv with a few test packages for VirtualenvTest to use.
 
     Returns the directory containing the "venv" virtualenv.
@@ -138,13 +142,12 @@ def venv_world_fixture(tmp_path_factory):
             """)
 
         # Install everything.
-        coverage_src = nice_file(TESTS_DIR, "..")
         run_in_venv(
-            "python -m pip install --no-index " +
+            "python -m pip install " +
             "./third_pkg " +
             "-e ./another_pkg " +
             "-e ./bug888/app -e ./bug888/plugin " +
-            coverage_src
+            COVERAGE_INSTALL_ARGS
         )
         shutil.rmtree("third_pkg")
 
@@ -155,24 +158,18 @@ def venv_world_fixture(tmp_path_factory):
     "coverage",
     "python -m coverage",
 ], name="coverage_command")
-def coverage_command_fixture(request):
+def coverage_command_fixture(request: pytest.FixtureRequest) -> str:
     """Parametrized fixture to use multiple forms of "coverage" command."""
-    return request.param
+    return cast(str, request.param)
 
 
-# https://bugs.python.org/issue46028
-@pytest.mark.xfail(
-    (3, 11, 0, 'alpha', 4, 0) == env.PYVERSION and
-        not os.path.exists(sys._base_executable),
-    reason="avoid 3.11 bug: bpo46028"
-)
 class VirtualenvTest(CoverageTest):
     """Tests of virtualenv considerations."""
 
     expected_stdout = "33\n110\n198\n1.5\n"
 
     @pytest.fixture(autouse=True)
-    def in_venv_world_fixture(self, venv_world):
+    def in_venv_world_fixture(self, venv_world: Path) -> Iterator[None]:
         """For running tests inside venv_world, and cleaning up made files."""
         with change_dir(venv_world):
             self.make_file("myproduct.py", """\
@@ -186,7 +183,7 @@ class VirtualenvTest(CoverageTest):
                 print(sum(colorsys.rgb_to_hls(1, 0, 0)))
                 """)
 
-            self.del_environ("COVERAGE_TESTING")    # To avoid needing contracts installed.
+            self.del_environ("COVERAGE_TESTING")    # To get realistic behavior
             self.set_environ("COVERAGE_DEBUG_FILE", "debug_out.txt")
             self.set_environ("COVERAGE_DEBUG", "trace")
 
@@ -196,12 +193,12 @@ class VirtualenvTest(CoverageTest):
                 if fname not in {"venv", "another_pkg", "bug888"}:
                     os.remove(fname)
 
-    def get_trace_output(self):
+    def get_trace_output(self) -> str:
         """Get the debug output of coverage.py"""
         with open("debug_out.txt") as f:
             return f.read()
 
-    def test_third_party_venv_isnt_measured(self, coverage_command):
+    def test_third_party_venv_isnt_measured(self, coverage_command: str) -> None:
         out = run_in_venv(coverage_command + " run --source=. myproduct.py")
         # In particular, this warning doesn't appear:
         # Already imported a file that will be measured: .../coverage/__main__.py
@@ -213,12 +210,12 @@ class VirtualenvTest(CoverageTest):
         assert re_lines(
             r"^Not tracing .*\bexecfile.py': inside --source, but is third-party",
             debug_out,
-            )
+        )
         assert re_lines(r"^Tracing .*\bmyproduct.py", debug_out)
         assert re_lines(
             r"^Not tracing .*\bcolorsys.py': falls outside the --source spec",
             debug_out,
-            )
+        )
 
         out = run_in_venv(coverage_command + " report")
         assert "myproduct.py" in out
@@ -226,7 +223,7 @@ class VirtualenvTest(CoverageTest):
         assert "coverage" not in out
         assert "colorsys" not in out
 
-    def test_us_in_venv_isnt_measured(self, coverage_command):
+    def test_us_in_venv_isnt_measured(self, coverage_command: str) -> None:
         out = run_in_venv(coverage_command + " run --source=third myproduct.py")
         assert out == self.expected_stdout
 
@@ -237,15 +234,15 @@ class VirtualenvTest(CoverageTest):
             r"^Not tracing .*\bexecfile.py': " +
             "module 'coverage.execfile' falls outside the --source spec",
             debug_out,
-            )
+        )
         assert re_lines(
             r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec",
             debug_out,
-            )
+        )
         assert re_lines(
             r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec",
             debug_out,
-            )
+        )
 
         out = run_in_venv(coverage_command + " report")
         assert "myproduct.py" not in out
@@ -253,7 +250,7 @@ class VirtualenvTest(CoverageTest):
         assert "coverage" not in out
         assert "colorsys" not in out
 
-    def test_venv_isnt_measured(self, coverage_command):
+    def test_venv_isnt_measured(self, coverage_command: str) -> None:
         out = run_in_venv(coverage_command + " run myproduct.py")
         assert out == self.expected_stdout
 
@@ -269,7 +266,7 @@ class VirtualenvTest(CoverageTest):
         assert "colorsys" not in out
 
     @pytest.mark.skipif(not env.C_TRACER, reason="Plugins are only supported with the C tracer.")
-    def test_venv_with_dynamic_plugin(self, coverage_command):
+    def test_venv_with_dynamic_plugin(self, coverage_command: str) -> None:
         # https://github.com/nedbat/coveragepy/issues/1150
         # Django coverage plugin was incorrectly getting warnings:
         # "Already imported: ... django/template/blah.py"
@@ -285,7 +282,7 @@ class VirtualenvTest(CoverageTest):
         # Already imported a file that will be measured: ...third/render.py (already-imported)
         assert out == "HTML: hello.html@1723\n"
 
-    def test_installed_namespace_packages(self, coverage_command):
+    def test_installed_namespace_packages(self, coverage_command: str) -> None:
         # https://github.com/nedbat/coveragepy/issues/1231
         # When namespace packages were installed, they were considered
         # third-party packages.  Test that isn't still happening.
@@ -301,15 +298,15 @@ class VirtualenvTest(CoverageTest):
             r"^Not tracing .*\bexecfile.py': " +
             "module 'coverage.execfile' falls outside the --source spec",
             debug_out,
-            )
+        )
         assert re_lines(
             r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec",
             debug_out,
-            )
+        )
         assert re_lines(
             r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec",
             debug_out,
-            )
+        )
 
         out = run_in_venv(coverage_command + " report")
 
@@ -327,10 +324,10 @@ class VirtualenvTest(CoverageTest):
         assert "fifth" in out
         assert "sixth" in out
 
-    def test_bug_888(self, coverage_command):
+    def test_bug_888(self, coverage_command: str) -> None:
         out = run_in_venv(
             coverage_command +
             " run --source=bug888/app,bug888/plugin bug888/app/testcov/main.py"
-            )
+        )
         # When the test fails, the output includes "Already imported a file that will be measured"
         assert out == "Plugin here\n"
